var express = require('express'),
    http = require('http'),
    path = require('path'),
    fs = require('fs'),
    jade = require('jade'),
    os = require('os'),
    socketio = require('socket.io'),
    deferred = require('deferred'),
    readFile = deferred.promisify(fs.readFile);

var TEMPLATE_FILE = './index.jade';
var LAYOUT_FILE = './layout.jade';
var ASSETS_TEMPLATE_FILE = __dirname + '/views/assets.jade';
var POPUP_CONTROLLER_TEMPLATE_FILE = __dirname + '/views/popup-controller.jade';
var POPUP_CONTROLLER_ASSETS_TEMPLATE = __dirname + '/views/popup-controller-assets.jade';

exports.start = start;

function start() {
    checkAssets();
    startServer();
}

function serveIndexPage(req, res) {
    deferred.map([TEMPLATE_FILE, ASSETS_TEMPLATE_FILE], function(fileName) {
        return readFile(fileName, 'utf-8');
    })
    .then(function(files) {
        var template = jade.compile(files[0], {filename: TEMPLATE_FILE});
        var assetsTemplate = jade.compile(files[1]);
        res.send(template({
            pres_assets: assetsTemplate()
        }));
    }).end();
}

function servePopupControllerPage(req, res) {
    deferred.map([POPUP_CONTROLLER_TEMPLATE_FILE, POPUP_CONTROLLER_ASSETS_TEMPLATE], function(fileName) {
        return readFile(fileName, 'utf-8');
    })
    .then(function(files) {
        var template = jade.compile(files[0], {filename: TEMPLATE_FILE});
        var assetsTemplate = jade.compile(files[1]);
        res.send(template({
            pres_assets: assetsTemplate()
        }));
    })
    .end();
}

function startServer() {
    var app = express();
    app.configure(function(){
        app.set('port', process.env.PORT || 3000);
        app.use(app.router);
        app.use(require('less-middleware')({ src: __dirname + '/public' }));
        app.use(express.static('./public'));
        app.use(express.static(path.join(__dirname, 'public')));
        app.use(express.errorHandler());
    });

    app.get('/', serveIndexPage);
    app.get('/master', serveIndexPage);
    app.get('/popup-controller', servePopupControllerPage);

    var server = http.createServer(app).listen(app.get('port'), function(err){
        console.log("==> Presentation running on http://" + os.hostname() + ':' + app.get('port'));
    }).on('error', function(error) {
        if(error.code === 'EADDRINUSE') {
            console.log('ERROR: Port '+app.get('port')+' is already in use.');
        } else {
            console.log('ERROR: ');
            console.log(error);
        }
    });
    var io = socketio.listen(server);
    io.set('log level', 0);
    var clientCount = 0;
    io.sockets.on('connection', function (socket) {
        clientCount++;
        console.log('- Client connected.');
        socket.on('master-slide-changed', function(data) {
            console.log('==> Master changed to slide ' + (data.currentSlide + 1));
            socket.broadcast.emit('slide-change', data);
            console.log('- Notified ' + (clientCount-1) + ' clients.');
        });
        socket.on('disconnect', function () {
            clientCount--;
            console.log('- Client disconnected.');
        });
    });
}

function checkAssets() {
    var templatesOK = true;
    [TEMPLATE_FILE, LAYOUT_FILE].forEach(function(file) {
        if(fs.existsSync(file)) {
            console.log('- Found '+file+'.');
        } else {
            templatesOK = false;
            console.log('- No '+file+' found in current directory.');
        }
    });
    if (!templatesOK) {
        process.exit(127);
    }
    if(fs.existsSync('public')) {
        console.log('- Serving extra assets from public/.');
    } else {
        console.log('- No public/ directory found for extra assets.');
    }
}
